@@ -1,7 +1,7 @@
 /*
  *  Copyright (C) 2003 Marco Pesenti Gritti
- *  Copyright (C) 2003 Christian Persch
- *  Copyright (C) 2003 Adam Hooper
+ *  Copyright (C) 2003, 2004 Christian Persch
+ *  Copyright (C) 2003, 2004 Adam Hooper
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-popup-blocker-extension.c,v 1.9 2004/03/08 23:00:58 adamh Exp $
+ *  $Id: ephy-popup-blocker-extension.c,v 1.19.2.4 2004/06/23 21:40:32 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -60,9 +60,9 @@ static void clear_popup_permissions (void);
 
 static GtkToggleActionEntry action_entries [] =
 {
-	{ "PopupBlocker", NULL, N_("Blo_ck Popups from this Site"),
+	{ "PopupBlocker", NULL, N_("Popup _Windows"),
 	  NULL, /* shortcut key */
-	  N_("Block or unblock popup windows"),
+	  N_("Show or hide unrequested popup windows from this site"),
 	  NULL, TRUE }
 };
 static const guint n_action_entries = G_N_ELEMENTS (action_entries);
@@ -116,8 +116,24 @@ ephy_popup_blocker_extension_register_type (GTypeModule *module)
 	return type;
 }
 
+EphyPopupBlockerIcon *
+get_icon_for_window (EphyWindow *window)
+{
+	GtkWidget *statusbar;
+	gpointer icon;
+
+	statusbar = ephy_window_get_statusbar (window);
+	g_return_val_if_fail (EPHY_IS_STATUSBAR (statusbar), NULL);
+
+	icon = g_object_get_data (G_OBJECT (statusbar), "popup-blocker-icon");
+	g_return_val_if_fail (EPHY_IS_POPUP_BLOCKER_ICON (icon), NULL);
+
+	return (EphyPopupBlockerIcon *) icon;
+}
+
 static void
-update_action (EphyWindow *window, const char *address)
+update_action (EphyWindow *window,
+	       const char *address)
 {
 	EphyPermissionManager *permission_manager;
 	GtkAction *action;
@@ -126,10 +142,11 @@ update_action (EphyWindow *window, const char *address)
 
 	LOG ("update_action: window %p, address %s", window, address)
 
-	permission_manager = EPHY_PERMISSION_MANAGER (ephy_embed_shell_get_embed_single (EPHY_EMBED_SHELL (ephy_shell)));
+	permission_manager = EPHY_PERMISSION_MANAGER
+		(ephy_embed_shell_get_embed_single (embed_shell));
 
-	response = ephy_permission_manager_test (permission_manager, address,
-						 EPT_POPUP);
+	response = ephy_permission_manager_test
+		(permission_manager, address, EPT_POPUP);
 
 	switch (response)
 	{
@@ -152,51 +169,35 @@ update_action (EphyWindow *window, const char *address)
 					 G_CALLBACK (action_activate_cb),
 					 window);
 
-	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
-				      allow == FALSE);
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), allow);
 
 	g_signal_handlers_unblock_by_func (G_OBJECT (action),
 					   G_CALLBACK (action_activate_cb),
 					   window);
 
-	if (allow == TRUE)
-	{
-		LOG ("Popups are allowed from %s", address);
-	}
-	else
-	{
-		LOG ("Popups are blocked from %s", address);
-	}
+	LOG ("Popups are %s from %s", allow ? "allowed" : "blocked", address);
 }
 
 static void
 update_action_without_address (EphyWindow *window)
 {
-	/*
-	 * FIXME: The address used to update_action could be inaccurate.
-	 * We can't use ephy_tab_get_embed() because of bug #119047
-	 */
+	EphyEmbed *embed;
+	char *address;
 
-	EphyTab *tab;
-	const char *address;
+	if (GTK_WIDGET_REALIZED (window) == FALSE) return;
 
-	tab = ephy_window_get_active_tab (window);
+	embed = ephy_window_get_active_embed (window);
 
-	if (tab == NULL) return; /* Happens on startup */
+	if (embed == NULL) return; /* Happens on startup */
 
-	g_return_if_fail (EPHY_IS_TAB (tab));
+	g_return_if_fail (EPHY_IS_EMBED (embed));
 
-	address = ephy_tab_get_location (tab);
+	address = mozilla_get_location (embed);
 	g_return_if_fail (address != NULL);
 
 	update_action (window, address);
-}
 
-static void
-update_all_actions_foreach_func (gpointer data,
-				 gpointer user_data)
-{
-	update_action_without_address (EPHY_WINDOW (data));
+	g_free (address);
 }
 
 static void
@@ -214,7 +215,8 @@ update_all_actions (GConfClient *client,
 	session = EPHY_SESSION (ephy_shell_get_session (ephy_shell));
 	windows = ephy_session_get_windows (session);
 
-	g_list_foreach (windows, (GFunc) update_all_actions_foreach_func, NULL);
+	g_list_foreach (windows, (GFunc) update_action_without_address, NULL);
+	g_list_free (windows);
 }
 
 static void
@@ -268,12 +270,9 @@ get_gconf_allow_popups_pref (void)
 }
 
 static void
-clear_one_permission_info (gpointer data,
-			   gpointer user_data)
+clear_one_permission_info (EphyPermissionInfo *info,
+			   EphyPermissionManager *manager)
 {
-	EphyPermissionInfo *info = (EphyPermissionInfo *) data;
-	EphyPermissionManager *manager = (EphyPermissionManager *) user_data;
-
 	ephy_permission_manager_remove (manager, info->host, EPT_POPUP);
 
 	ephy_permission_info_free (info);
@@ -285,56 +284,23 @@ clear_popup_permissions (void)
 	EphyPermissionManager *manager;
 	GList *list;
 
-	manager = EPHY_PERMISSION_MANAGER (ephy_embed_shell_get_embed_single (EPHY_EMBED_SHELL (ephy_shell)));
+	manager = EPHY_PERMISSION_MANAGER
+		(ephy_embed_shell_get_embed_single (embed_shell));
 
 	list = ephy_permission_manager_list (manager, EPT_POPUP);
 
 	g_list_foreach (list, (GFunc) clear_one_permission_info, manager);
-
 	g_list_free (list);
 }
 
-EphyPopupBlockerIcon *
-get_icon_for_embed (EphyEmbed *embed)
-{
-	EphyWindow *window;
-	EphyTab *tab;
-	GtkWidget *statusbar;
-	gpointer icon;
-
-	g_return_val_if_fail (EPHY_IS_EMBED (embed), NULL);
-
-	tab = ephy_tab_for_embed (embed);
-	g_return_val_if_fail (EPHY_IS_TAB (tab), NULL);
-
-	window = ephy_tab_get_window (tab);
-	g_return_val_if_fail (EPHY_IS_WINDOW (window), NULL);
-
-	statusbar = ephy_window_get_statusbar (window);
-	g_return_val_if_fail (EPHY_IS_STATUSBAR (statusbar), NULL);
-
-	icon = g_object_get_data (G_OBJECT (statusbar), "popup-blocker-icon");
-	g_return_val_if_fail (EPHY_IS_POPUP_BLOCKER_ICON (icon), NULL);
-
-	return (EphyPopupBlockerIcon *) icon;
-}
-
 static void
 location_cb (EphyEmbed *embed,
 	     const char *address,
-	     EphyTab *tab)
+	     EphyWindow *window)
 {
-	EphyWindow *window;
 	GtkWidget *statusbar;
 	EphyPopupBlockerList *popups;
 
-	if (tab == NULL || embed == NULL) return;
-	g_return_if_fail (EPHY_IS_TAB (tab));
-	g_return_if_fail (EPHY_IS_EMBED (embed));
-
-	window = ephy_tab_get_window (tab);
-	g_return_if_fail (EPHY_IS_WINDOW (window));
-
 	update_action (window, address);
 
 	popups = g_object_get_data (G_OBJECT (embed), "popup-blocker-list");
@@ -364,25 +330,30 @@ action_activate_cb (GtkAction *action,
 		    EphyWindow *window)
 {
 	const char *address;
-	EphyTab *tab;
+	EphyEmbed *embed;
+	EphyPopupBlockerList *list;
 	EphyPermissionManager *permission_manager;
 	EphyPermission allow;
 
-	tab = ephy_window_get_active_tab (window);
-	g_return_if_fail (EPHY_IS_TAB (tab));
+	embed = ephy_window_get_active_embed (window);
+	g_return_if_fail (EPHY_IS_EMBED (embed));
 
-	address = ephy_tab_get_location (tab);
+	address = ephy_embed_get_location (embed, TRUE);
 	g_return_if_fail (address != NULL);
 
-	permission_manager = EPHY_PERMISSION_MANAGER (ephy_embed_shell_get_embed_single (EPHY_EMBED_SHELL (ephy_shell)));
+	list = g_object_get_data (G_OBJECT (embed), "popup-blocker-list");
+	g_return_if_fail (EPHY_IS_POPUP_BLOCKER_LIST (list));
+
+	permission_manager = EPHY_PERMISSION_MANAGER
+		(ephy_embed_shell_get_embed_single (embed_shell));
 
 	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)) == TRUE)
 	{
-		allow = EPHY_PERMISSION_DENIED;
+		allow = EPHY_PERMISSION_ALLOWED;
 	}
 	else
 	{
-		allow = EPHY_PERMISSION_ALLOWED;
+		allow = EPHY_PERMISSION_DENIED;
 	}
 
 	LOG ("from now on, address '%s' will %s popups", address,
@@ -390,6 +361,15 @@ action_activate_cb (GtkAction *action,
 
 	ephy_permission_manager_add (permission_manager, address,
 				     EPT_POPUP, allow); 
+
+	if (allow == EPHY_PERMISSION_ALLOWED)
+	{
+		ephy_popup_blocker_list_show_all (list);
+	}
+	else
+	{
+		ephy_popup_blocker_list_hide_all (list);
+	}
 }
 
 static void
@@ -410,12 +390,14 @@ create_statusbar_icon (EphyWindow *window)
 }
 
 static void
-unregister_mozilla (PopupListenerFreeData *data)
+unregister_mozilla (GtkObject *embed, PopupListenerFreeData *data)
 {
-	LOG ("unregister_mozilla with freeing data %x", data)
+	LOG ("unregister_mozilla with freeing data %p", data)
 
 	g_return_if_fail (data != NULL);
 
+	g_object_set_data (G_OBJECT (embed),  "popup-blocker-listener-data", NULL);
+
 	mozilla_unregister_popup_listener (data);
 }
 
@@ -436,23 +418,47 @@ register_mozilla (EphyEmbed *embed)
 			(embed, G_CALLBACK (register_mozilla), NULL);
 	}
 
-	LOG ("register_mozilla with EphyEmbed at %x", embed)
+	LOG ("register_mozilla with EphyEmbed at %p", embed)
 
 	g_return_if_fail (EPHY_IS_EMBED (embed));
 
+	if (g_object_get_data (G_OBJECT (embed), "popup-blocker-listener-data"))
+	{
+		return;
+	}
+
 	data = mozilla_register_popup_listener (embed);
 	g_return_if_fail (data != NULL);
 
-	LOG ("Registered listener; freeing data is at %x", data)
+	LOG ("Registered listener; freeing data is at %p", data)
 
-	g_object_set_data_full (G_OBJECT (embed), "popup-blocker-listener-data",
-				data, (GDestroyNotify) unregister_mozilla);
+	g_signal_connect (embed, "destroy",
+			  G_CALLBACK (unregister_mozilla), data);
+
+	g_object_set_data (G_OBJECT (embed), "popup-blocker-listener-data", data);
+}
+
+static void
+new_window_cb (EphyEmbed *embed,
+	       EphyEmbed **new_embed,
+	       EphyEmbedChrome chromemask,
+	       EphyPopupBlockerList *popups)
+{
+	GtkWidget *toplevel;
+
+	if (*new_embed == NULL) return;
+	g_return_if_fail (EPHY_IS_EMBED (*new_embed));
+
+	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (*new_embed));
+	g_return_if_fail (toplevel != NULL);
+
+	ephy_popup_blocker_list_insert_window (popups, EPHY_WINDOW (toplevel));
 }
 
 static void
 tab_added_cb (GtkWidget *notebook,
 	      EphyTab *tab,
-	      EphyPopupBlockerExtension *extension)
+	      EphyWindow *window)
 {
 	EphyEmbed *embed;
 	EphyPopupBlockerList *popups;
@@ -460,74 +466,87 @@ tab_added_cb (GtkWidget *notebook,
 
 	g_return_if_fail (EPHY_IS_TAB (tab));
 
+	LOG ("tab_added_cb: tab %p, window %p", tab, window)
+
 	embed = ephy_tab_get_embed (tab);
 	g_return_if_fail (EPHY_IS_EMBED (embed));
 
-	popups = ephy_popup_blocker_list_new ();
-	g_return_if_fail (EPHY_IS_POPUP_BLOCKER_LIST (popups));
+	popups = g_object_get_data (G_OBJECT (embed), "popup-blocker-list");
 
-	g_object_set_data_full (G_OBJECT (embed), "popup-blocker-list", popups,
-				(GDestroyNotify) g_object_unref);
+	if (popups == NULL)
+	{
+		popups = ephy_popup_blocker_list_new (embed);
+		g_return_if_fail (EPHY_IS_POPUP_BLOCKER_LIST (popups));
 
-	icon = get_icon_for_embed (embed);
-	g_return_if_fail (EPHY_IS_POPUP_BLOCKER_ICON (icon));
+		g_object_set_data_full (G_OBJECT (embed),
+					"popup-blocker-list", popups,
+					(GDestroyNotify) g_object_unref);
+	}
+
+	icon = get_icon_for_window (window);
+	g_return_if_fail (icon != NULL);
 
 	ephy_popup_blocker_icon_set_popups (icon, popups);
 
 	register_mozilla (embed);
 
 	g_signal_connect (embed, "ge_location",
-			  G_CALLBACK (location_cb), tab);
+			  G_CALLBACK (location_cb), window);
+	g_signal_connect_object (embed, "ge_new_window",
+				 G_CALLBACK (new_window_cb), popups,
+				 G_CONNECT_AFTER);
 }
 
 static void
 tab_removed_cb (GtkWidget *notebook,
 		EphyTab *tab,
-		EphyPopupBlockerExtension *extension)
+		EphyWindow *window)
 {
 	EphyEmbed *embed;
-	EphyPopupBlockerIcon *icon;
+	EphyPopupBlockerList *popups;
 
 	g_return_if_fail (EPHY_IS_TAB (tab));
 
+	LOG ("tab_removed_cb: tab %p, window %p\n", tab, window)
+
 	embed = ephy_tab_get_embed (tab);
 	g_return_if_fail (EPHY_IS_EMBED (embed));
 
-	icon = get_icon_for_embed (embed);
-	g_return_if_fail (EPHY_IS_POPUP_BLOCKER_ICON (icon));
+	popups = g_object_get_data (G_OBJECT (embed), "popup-blocker-list");
+	g_return_if_fail (EPHY_IS_POPUP_BLOCKER_LIST (popups));
 
 	g_signal_handlers_disconnect_by_func
-		(embed, G_CALLBACK (location_cb), tab);
+		(embed, G_CALLBACK (location_cb), window);
+	g_signal_handlers_disconnect_by_func
+		(embed, G_CALLBACK (new_window_cb), popups);
 }
 
 static void
-switch_page_cb (GtkNotebook *notebook,
-		GtkNotebookPage *page,
-		guint page_num,
-		EphyPopupBlockerExtension *extension)
+sync_active_tab_cb (EphyWindow *window,
+		    GParamSpec *pspec,
+		    gpointer dummy)
 {
-	GtkWidget *toplevel;
 	EphyEmbed *embed;
 	EphyPopupBlockerIcon *icon;
 	EphyPopupBlockerList *popups;
 
-	LOG ("switch_page_cb: page %x", page)
+	LOG ("sync_active_tab window %p", window)
 
-	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (notebook));
-	g_return_if_fail (EPHY_IS_WINDOW (toplevel));
-	if (GTK_WIDGET_REALIZED (toplevel) == FALSE) return; /* on startup */
+	g_return_if_fail (EPHY_IS_WINDOW (window));
 
-	embed = ephy_window_get_active_embed (EPHY_WINDOW (toplevel));
+	if (GTK_WIDGET_REALIZED (window) == FALSE) return;
+
+	embed = ephy_window_get_active_embed (window);
 	g_return_if_fail (EPHY_IS_EMBED (embed));
 
-	icon = get_icon_for_embed (embed);
-	g_return_if_fail (EPHY_IS_POPUP_BLOCKER_ICON (icon));
+	icon = get_icon_for_window (window);
+	g_return_if_fail (icon != NULL);
 
 	popups = g_object_get_data (G_OBJECT (embed), "popup-blocker-list");
 
 	ephy_popup_blocker_icon_set_popups (icon, popups);
 
-	update_action_without_address (EPHY_WINDOW (toplevel));
+	update_action_without_address (window);
 }
 
 static void
@@ -559,10 +578,10 @@ impl_attach_window (EphyExtension *extension,
 
 	gtk_ui_manager_add_ui (manager, merge_id, "/menubar/ViewMenu",
 			       "PopupBlockerSep", NULL,
-			       GTK_UI_MANAGER_SEPARATOR, TRUE);
+			       GTK_UI_MANAGER_SEPARATOR, FALSE);
 	gtk_ui_manager_add_ui (manager, merge_id, "/menubar/ViewMenu",
 			       "PopupBlocker", "PopupBlocker",
-			       GTK_UI_MANAGER_MENUITEM, TRUE);
+			       GTK_UI_MANAGER_MENUITEM, FALSE);
 
 	/*
 	 * Connect the GtkToggleAction's activate signal
@@ -576,16 +595,17 @@ impl_attach_window (EphyExtension *extension,
 	g_signal_connect_after (action, "activate",
 				G_CALLBACK (action_activate_cb), window);
 
+	g_signal_connect (window, "notify::active-tab",
+			  G_CALLBACK (sync_active_tab_cb), NULL);
+
 	/* Notebook signals */
 
 	notebook = ephy_window_get_notebook (window);
 
 	g_signal_connect_after (notebook, "tab_added",
-				G_CALLBACK (tab_added_cb), extension);
+				G_CALLBACK (tab_added_cb), window);
 	g_signal_connect_after (notebook, "tab_removed",
-				G_CALLBACK (tab_removed_cb), extension);
-	g_signal_connect_after (notebook, "switch_page",
-				G_CALLBACK (switch_page_cb), extension);
+				G_CALLBACK (tab_removed_cb), window);
 }
 
 static void
@@ -596,14 +616,15 @@ impl_detach_window (EphyExtension *extension,
 
 	g_return_if_fail (EPHY_IS_WINDOW (window));
 
+	g_signal_handlers_disconnect_by_func
+		(window, G_CALLBACK (sync_active_tab_cb), window);
+
 	notebook = ephy_window_get_notebook (window);
 
 	g_signal_handlers_disconnect_by_func
-		(notebook, G_CALLBACK (tab_added_cb), extension);
+		(notebook, G_CALLBACK (tab_added_cb), window);
 	g_signal_handlers_disconnect_by_func
-		(notebook, G_CALLBACK (tab_removed_cb), extension);
-	g_signal_handlers_disconnect_by_func
-		(notebook, G_CALLBACK (switch_page_cb), extension);
+		(notebook, G_CALLBACK (tab_removed_cb), window);
 }
 
 static void
